@@ -78,6 +78,7 @@ static QString realSystemPath()
 void JobTest::initTestCase()
 {
     QStandardPaths::enableTestMode(true);
+    QCoreApplication::instance()->setApplicationName("kio/jobtest"); // testing for #357499
 
     // To avoid a runtime dependency on klauncher
     qputenv("KDE_FORK_SLAVES", "yes");
@@ -246,6 +247,28 @@ void JobTest::storedPutIODevice()
     QVERIFY(!spyPercent.isEmpty());
 }
 
+void JobTest::storedPutIODeviceFile()
+{
+    // Given a source file and a destination file
+    const QString src = homeTmpDir() + "fileFromHome";
+    createTestFile(src);
+    QVERIFY(QFile::exists(src));
+    QFile srcFile(src);
+    QVERIFY(srcFile.open(QIODevice::ReadOnly));
+    const QString dest = homeTmpDir() + "fileFromHome_copied";
+    QFile::remove(dest);
+    const QUrl destUrl = QUrl::fromLocalFile(dest);
+
+    // When using storedPut with the QFile as argument
+    KIO::StoredTransferJob *job = KIO::storedPut(&srcFile, destUrl, 0600, KIO::Overwrite | KIO::HideProgressInfo);
+
+    // Then the copy should succeed and the dest file exist
+    QVERIFY2(job->exec(), qPrintable(job->errorString()));
+    QVERIFY(QFile::exists(dest));
+    QCOMPARE(QFileInfo(src).size(), QFileInfo(dest).size());
+    QFile::remove(dest);
+}
+
 void JobTest::storedPutIODeviceSlowDevice()
 {
     const QString filePath = homeTmpDir() + "fileFromHome";
@@ -341,6 +364,15 @@ void JobTest::copyLocalFile(const QString &src, const QString &dest)
 #endif
     }
     QCOMPARE(spyCopyingDone.count(), 1);
+
+    // cleanup and retry with KIO::copyAs()
+    QFile::remove(dest);
+    job = KIO::copyAs(u, d, KIO::HideProgressInfo);
+    job->setUiDelegate(0);
+    job->setUiDelegateExtension(0);
+    QVERIFY(job->exec());
+    QVERIFY(QFile::exists(dest));
+    QVERIFY(QFile::exists(src));     // still there
 }
 
 void JobTest::copyLocalDirectory(const QString &src, const QString &_dest, int flags)
@@ -402,7 +434,7 @@ static QString linkTarget(const QString &path)
 {
     // Use readlink on Unix because symLinkTarget turns relative targets into absolute (#352927)
     char linkTargetBuffer[4096];
-    const int n = readlink(QFile::encodeName(path).constData(), linkTargetBuffer, sizeof(linkTargetBuffer)-1);
+    const int n = readlink(QFile::encodeName(path).constData(), linkTargetBuffer, sizeof(linkTargetBuffer) - 1);
     if (n != -1) {
         linkTargetBuffer[n] = 0;
     }
@@ -518,19 +550,19 @@ void JobTest::copyFolderWithUnaccessibleSubfolder()
     QFile(inaccessible).setPermissions(QFile::Permissions()); // Make it inaccessible
     //Copying should throw some warnings, as it cannot access some folders
 
-    KIO::CopyJob* job = KIO::copy(QUrl::fromLocalFile(src_dir), QUrl::fromLocalFile(dst_dir), KIO::HideProgressInfo);
+    KIO::CopyJob *job = KIO::copy(QUrl::fromLocalFile(src_dir), QUrl::fromLocalFile(dst_dir), KIO::HideProgressInfo);
 
     QSignalSpy spy(job, SIGNAL(warning(KJob*,QString,QString)));
-    job->setUiDelegate( 0 ); // no skip dialog, thanks
+    job->setUiDelegate(0);   // no skip dialog, thanks
     QVERIFY(job->exec());
 
     QFile(inaccessible).setPermissions(QFile::Permissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner));
 
-    KIO::DeleteJob* deljob1 = KIO::del(QUrl::fromLocalFile(src_dir), KIO::HideProgressInfo);
+    KIO::DeleteJob *deljob1 = KIO::del(QUrl::fromLocalFile(src_dir), KIO::HideProgressInfo);
     deljob1->setUiDelegate(0); // no skip dialog, thanks
     QVERIFY(deljob1->exec());
 
-    KIO::DeleteJob* deljob2 = KIO::del(QUrl::fromLocalFile(dst_dir), KIO::HideProgressInfo);
+    KIO::DeleteJob *deljob2 = KIO::del(QUrl::fromLocalFile(dst_dir), KIO::HideProgressInfo);
     deljob2->setUiDelegate(0); // no skip dialog, thanks
     QVERIFY(deljob2->exec());
 
@@ -685,60 +717,60 @@ void JobTest::moveDirectoryToOtherPartition()
 
 void JobTest::moveFileNoPermissions()
 {
-    qDebug();
-#ifdef Q_OS_WIN
-    qDebug() << "port to win32";
-#else
-    const QString src = QStringLiteral("/etc/passwd");
-    const QString dest = homeTmpDir() + "passwd";
-    QVERIFY(QFile::exists(src));
-    QVERIFY(QFileInfo(src).isFile());
-    QUrl u = QUrl::fromLocalFile(src);
-    QUrl d = QUrl::fromLocalFile(dest);
+    // Given a file that cannot be moved (subdir has no permissions)
+    const QString subdir = homeTmpDir() + "subdir";
+    QVERIFY(QDir().mkpath(subdir));
+    const QString src = subdir + "/thefile";
+    createTestFile(src);
+    QVERIFY(QFile(subdir).setPermissions(QFile::Permissions())); // Make it inaccessible
 
-    KIO::CopyJob *job = KIO::move(u, d, KIO::HideProgressInfo);
+    // When trying to move it
+    const QString dest = homeTmpDir() + "dest";
+    KIO::CopyJob *job = KIO::move(QUrl::fromLocalFile(src), QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
     job->setUiDelegate(0);
     job->setUiDelegateExtension(0); // no skip dialog, thanks
-    bool ok = job->exec();
-    QVERIFY(!ok);
+
+    // The job should fail with "access denied"
+    QVERIFY(!job->exec());
     QCOMPARE(job->error(), (int)KIO::ERR_ACCESS_DENIED);
-    // OK this is fishy. Just like mv(1), KIO's behavior depends on whether
+    // Note that, just like mv(1), KIO's behavior depends on whether
     // a direct rename(2) was used, or a full copy+del. In the first case
     // there is no destination file created, but in the second case the
     // destination file remains.
-    //QVERIFY( QFile::exists( dest ) );
+    // In this test it's the same partition, so no dest created.
+    QVERIFY(!QFile::exists(dest));
+
+    // Cleanup
+    QVERIFY(QFile(subdir).setPermissions(QFile::Permissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner)));
     QVERIFY(QFile::exists(src));
-#endif
+    QVERIFY(QDir(subdir).removeRecursively());
 }
 
 void JobTest::moveDirectoryNoPermissions()
 {
-    qDebug();
-#ifdef Q_OS_WIN
-    qDebug() << "port to win32";
-#else
-
-    // All of /etc is a bit much, so try to find something smaller:
-    QString src = QStringLiteral("/etc/fonts");
-    if (!QFile::exists(src)) {
-        src = QStringLiteral("/etc");
-    }
-
-    const QString dest = homeTmpDir() + "mdnp";
-    QVERIFY(QFile::exists(src));
+    // Given a dir that cannot be moved (parent dir has no permissions)
+    const QString subdir = homeTmpDir() + "subdir";
+    const QString src = subdir + "/thedir";
+    QVERIFY(QDir().mkpath(src));
     QVERIFY(QFileInfo(src).isDir());
-    QUrl u = QUrl::fromLocalFile(src);
-    QUrl d = QUrl::fromLocalFile(dest);
+    QVERIFY(QFile(subdir).setPermissions(QFile::Permissions())); // Make it inaccessible
 
-    KIO::CopyJob *job = KIO::move(u, d, KIO::HideProgressInfo);
+    // When trying to move it
+    const QString dest = homeTmpDir() + "mdnp";
+    KIO::CopyJob *job = KIO::move(QUrl::fromLocalFile(src), QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
     job->setUiDelegate(0);
     job->setUiDelegateExtension(0); // no skip dialog, thanks
-    bool ok = job->exec();
-    QVERIFY(!ok);
+
+    // The job should fail with "access denied"
+    QVERIFY(!job->exec());
     QCOMPARE(job->error(), (int)KIO::ERR_ACCESS_DENIED);
-    //QVERIFY( QFile::exists( dest ) ); // see moveFileNoPermissions
+
+    QVERIFY(!QFile::exists(dest));
+
+    // Cleanup
+    QVERIFY(QFile(subdir).setPermissions(QFile::Permissions(QFile::ReadOwner | QFile::WriteOwner | QFile::ExeOwner)));
     QVERIFY(QFile::exists(src));
-#endif
+    QVERIFY(QDir(subdir).removeRecursively());
 }
 
 void JobTest::listRecursive()
@@ -1187,7 +1219,8 @@ void JobTest::stat()
 #if 1
     const QString filePath = homeTmpDir() + "fileFromHome";
     createTestFile(filePath);
-    KIO::StatJob *job = KIO::stat(QUrl::fromLocalFile(filePath), KIO::HideProgressInfo);
+    const QUrl url(QUrl::fromLocalFile(filePath));
+    KIO::StatJob *job = KIO::stat(url, KIO::HideProgressInfo);
     QVERIFY(job);
     bool ok = job->exec();
     QVERIFY(ok);
@@ -1195,7 +1228,21 @@ void JobTest::stat()
     const KIO::UDSEntry &entry = job->statResult();
     QVERIFY(!entry.isDir());
     QVERIFY(!entry.isLink());
-    QCOMPARE(entry.stringValue(KIO::UDSEntry::UDS_NAME), QString("fileFromHome"));
+    QCOMPARE(entry.stringValue(KIO::UDSEntry::UDS_NAME), QStringLiteral("fileFromHome"));
+
+    // Compare what we get via kio_file and what we get when KFileItem stat()s directly
+    const KFileItem kioItem(entry, url);
+    const KFileItem fileItem(url);
+    QCOMPARE(kioItem.name(), fileItem.name());
+    QCOMPARE(kioItem.url(), fileItem.url());
+    QCOMPARE(kioItem.size(), fileItem.size());
+    QCOMPARE(kioItem.user(), fileItem.user());
+    QCOMPARE(kioItem.group(), fileItem.group());
+    QCOMPARE(kioItem.mimetype(), fileItem.mimetype());
+    QCOMPARE(kioItem.permissions(), fileItem.permissions());
+    QCOMPARE(kioItem.time(KFileItem::ModificationTime), fileItem.time(KFileItem::ModificationTime));
+    QCOMPARE(kioItem.time(KFileItem::AccessTime), fileItem.time(KFileItem::AccessTime));
+
 #else
     // Testing stat over HTTP
     KIO::StatJob *job = KIO::stat(QUrl("http://www.kde.org"), KIO::HideProgressInfo);
@@ -1210,6 +1257,45 @@ void JobTest::stat()
 #endif
 }
 
+#ifndef Q_OS_WIN
+void JobTest::statSymlink()
+{
+    const QString filePath = homeTmpDir() + "fileFromHome";
+    createTestFile(filePath);
+    const QString symlink = otherTmpDir() + "link";
+    QVERIFY(QFile(filePath).link(symlink));
+    QVERIFY(QFile::exists(symlink));
+    setTimeStamp(symlink, QDateTime::currentDateTime().addSecs(-20)); // differenciate link time and source file time
+
+    const QUrl url(QUrl::fromLocalFile(symlink));
+    KIO::StatJob *job = KIO::stat(url, KIO::HideProgressInfo);
+    QVERIFY(job);
+    bool ok = job->exec();
+    QVERIFY(ok);
+    // TODO set setSide, setDetails
+    const KIO::UDSEntry &entry = job->statResult();
+    QVERIFY(!entry.isDir());
+    QVERIFY(entry.isLink());
+    QCOMPARE(entry.stringValue(KIO::UDSEntry::UDS_NAME), QStringLiteral("link"));
+
+    // Compare what we get via kio_file and what we get when KFileItem stat()s directly
+    const KFileItem kioItem(entry, url);
+    const KFileItem fileItem(url);
+    QCOMPARE(kioItem.name(), fileItem.name());
+    QCOMPARE(kioItem.url(), fileItem.url());
+    QVERIFY(kioItem.isLink());
+    QVERIFY(fileItem.isLink());
+    QCOMPARE(kioItem.linkDest(), fileItem.linkDest());
+    QCOMPARE(kioItem.size(), fileItem.size());
+    QCOMPARE(kioItem.user(), fileItem.user());
+    QCOMPARE(kioItem.group(), fileItem.group());
+    QCOMPARE(kioItem.mimetype(), fileItem.mimetype());
+    QCOMPARE(kioItem.permissions(), fileItem.permissions());
+    QCOMPARE(kioItem.time(KFileItem::ModificationTime), fileItem.time(KFileItem::ModificationTime));
+    QCOMPARE(kioItem.time(KFileItem::AccessTime), fileItem.time(KFileItem::AccessTime));
+}
+#endif
+
 void JobTest::mostLocalUrl()
 {
     const QString filePath = homeTmpDir() + "fileFromHome";
@@ -1276,7 +1362,7 @@ void JobTest::mimeType()
     QVERIFY(ok);
     QCOMPARE(spyMimeType.count(), 1);
     QCOMPARE(spyMimeType[0][0], QVariant::fromValue(static_cast<KIO::Job *>(job)));
-    QCOMPARE(spyMimeType[0][1].toString(), QString("application/octet-stream"));
+    QCOMPARE(spyMimeType[0][1].toString(), QStringLiteral("application/octet-stream"));
 #else
     // Testing mimetype over HTTP
     KIO::MimetypeJob *job = KIO::mimetype(QUrl("http://www.kde.org"), KIO::HideProgressInfo);
@@ -1359,7 +1445,7 @@ void JobTest::moveDestAlreadyExistsAutoRename()
 
 void JobTest::moveDestAlreadyExistsAutoRename(const QString &destDir, bool moveDirs) // #256650
 {
-    const QString prefix = moveDirs ? "dir " : "file ";
+    const QString prefix = moveDirs ? QStringLiteral("dir ") : QStringLiteral("file ");
     QStringList sources;
     const QString file1 = homeTmpDir() + prefix + "(1)";
     const QString file2 = homeTmpDir() + prefix + "(2)";
@@ -1473,6 +1559,59 @@ void JobTest::moveOverSymlinkToSelf() // #169547
 #endif
 }
 
+void JobTest::createSymlink()
+{
+#ifdef Q_OS_WIN
+    QSKIP("Test skipped on Windows");
+#endif
+    const QString sourceFile = homeTmpDir() + "fileFromHome";
+    createTestFile(sourceFile);
+    const QString destDir = homeTmpDir() + "dest";
+    QVERIFY(QDir().mkpath(destDir));
+
+    KIO::CopyJob *job = KIO::link(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(destDir), KIO::HideProgressInfo);
+    QVERIFY(job->exec());
+    QVERIFY(QFileInfo::exists(sourceFile));
+    const QString dest = destDir + "/fileFromHome";
+    QVERIFY(QFileInfo(dest).isSymLink());
+    QVERIFY(QDir(destDir).removeRecursively());
+}
+
+void JobTest::createSymlinkAsShouldSucceed()
+{
+#ifdef Q_OS_WIN
+    QSKIP("Test skipped on Windows");
+#endif
+    const QString sourceFile = homeTmpDir() + "fileFromHome";
+    createTestFile(sourceFile);
+    const QString dest = homeTmpDir() + "testlink";
+    QFile::remove(dest); // just in case
+
+    KIO::CopyJob *job = KIO::linkAs(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
+    QVERIFY(job->exec());
+    QVERIFY(QFileInfo::exists(sourceFile));
+    QVERIFY(QFileInfo(dest).isSymLink());
+    QVERIFY(QFile::remove(dest));
+}
+
+void JobTest::createSymlinkAsShouldFail()
+{
+#ifdef Q_OS_WIN
+    QSKIP("Test skipped on Windows");
+#endif
+    const QString sourceFile = homeTmpDir() + "fileFromHome";
+    createTestFile(sourceFile);
+    const QString dest = homeTmpDir() + "dest";
+    QVERIFY(QDir().mkpath(dest)); // dest exists as a directory
+
+    KIO::CopyJob *job = KIO::linkAs(QUrl::fromLocalFile(sourceFile), QUrl::fromLocalFile(dest), KIO::HideProgressInfo);
+    QVERIFY(!job->exec());
+    QCOMPARE(job->error(), (int)KIO::ERR_DIR_ALREADY_EXIST);
+    QVERIFY(QFileInfo::exists(sourceFile));
+    QVERIFY(!QFileInfo::exists(dest + "/fileFromHome"));
+    QVERIFY(QDir().rmdir(dest));
+}
+
 void JobTest::multiGet()
 {
     const int numFiles = 10;
@@ -1482,8 +1621,8 @@ void JobTest::multiGet()
 
     //qDebug() << file;
     KIO::MultiGetJob *job = KIO::multi_get(0, urls.at(0), KIO::MetaData()); // TODO: missing KIO::HideProgressInfo
-    QSignalSpy spyData(job, SIGNAL(data(long, QByteArray)));
-    QSignalSpy spyMimeType(job, SIGNAL(mimetype(long, QString)));
+    QSignalSpy spyData(job, SIGNAL(data(long,QByteArray)));
+    QSignalSpy spyMimeType(job, SIGNAL(mimetype(long,QString)));
     QSignalSpy spyResultId(job, SIGNAL(result(long)));
     QSignalSpy spyResult(job, SIGNAL(result(KJob*)));
     job->setUiDelegate(0);
@@ -1505,9 +1644,9 @@ void JobTest::multiGet()
     for (int i = 0; i < numFiles; ++i) {
         QCOMPARE(spyResultId.at(i).at(0).toInt(), i);
         QCOMPARE(spyMimeType.at(i).at(0).toInt(), i);
-        QCOMPARE(spyMimeType.at(i).at(1).toString(), QString("text/plain"));
+        QCOMPARE(spyMimeType.at(i).at(1).toString(), QStringLiteral("text/plain"));
         QCOMPARE(spyData.at(i * 2).at(0).toInt(), i);
-        QCOMPARE(QString(spyData.at(i * 2).at(1).toByteArray()), QString("Hello"));
+        QCOMPARE(QString(spyData.at(i * 2).at(1).toByteArray()), QStringLiteral("Hello"));
         QCOMPARE(spyData.at(i * 2 + 1).at(0).toInt(), i);
         QCOMPARE(QString(spyData.at(i * 2 + 1).at(1).toByteArray()), QLatin1String(""));
     }
