@@ -38,11 +38,8 @@
 #include <kurlauthorized.h>
 #include <kuser.h>
 
-#define QL1S(x) QLatin1String(x)
-#define QL1C(x) QLatin1Char(x)
-
 namespace {
-QLoggingCategory category("org.kde.kurifilter-shorturi");
+QLoggingCategory category("org.kde.kurifilter-shorturi", QtWarningMsg);
 }
 
 /**
@@ -89,7 +86,7 @@ static QString removeArgs( const QString& _cmd )
     if( spacePos > 0 )
     {
       cmd = cmd.left( spacePos );
-      //qCDebug(category) << "spacePos=" << spacePos << " returning " << cmd;
+      qCDebug(category) << "spacePos=" << spacePos << " returning " << cmd;
     }
   }
 
@@ -98,7 +95,7 @@ static QString removeArgs( const QString& _cmd )
 
 static bool isKnownProtocol(const QString &protocol)
 {
-    if (KProtocolInfo::isKnownProtocol(protocol)) {
+    if (KProtocolInfo::isKnownProtocol(protocol) || protocol == QLatin1String("mailto")) {
         return true;
     }
     const KService::Ptr service = KMimeTypeTrader::self()->preferredService(QLatin1String("x-scheme-handler/") + protocol);
@@ -167,11 +164,11 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
   // Fix misparsing of "foo:80", QUrl thinks "foo" is the protocol and "80" is the path.
   // However, be careful not to do that for valid hostless URLs, e.g. file:///foo!
   if (!protocol.isEmpty() && url.host().isEmpty() && !url.path().isEmpty()
-      && cmd.contains(':') && !KProtocolInfo::protocols().contains(protocol)) {
+      && cmd.contains(':') && !isKnownProtocol(protocol)) {
     protocol.clear();
   }
 
-  //qCDebug(category) << "url=" << url << "cmd=" << cmd << "isMalformed=" << isMalformed;
+  qCDebug(category) << "url=" << url << "cmd=" << cmd << "isMalformed=" << isMalformed;
 
   // TODO: Make this a bit more intelligent for Minicli! There
   // is no need to make comparisons if the supplied data is a local
@@ -227,11 +224,11 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
 
   if (QDir::isRelativePath(cmd) && QUrl(cmd).isRelative()) {
      path = cmd;
-     //qCDebug(category) << "path=cmd=" << path;
+     qCDebug(category) << "path=cmd=" << path;
   } else {
     if (url.isLocalFile())
     {
-      //qCDebug(category) << "hasRef=" << url.hasFragment();
+      qCDebug(category) << "hasRef=" << url.hasFragment();
       // Split path from ref/query
       // but not for "/tmp/a#b", if "a#b" is an existing file,
       // or for "/tmp/a?b" (#58990)
@@ -240,8 +237,7 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
       {
         path = url.path();
         ref = url.fragment();
-        //qCDebug(category) << "isLocalFile set path to " << stringDetails( path );
-        //qCDebug(category) << "isLocalFile set ref to " << stringDetails( ref );
+        qCDebug(category) << "isLocalFile set path to" << path << "and ref to" << ref;
         query = url.query();
         if (path.isEmpty() && !url.host().isEmpty())
           path = '/';
@@ -315,7 +311,7 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
       {
         ref = path.mid( pos + 1 );
         path = newPath;
-        //qCDebug(category) << "Extracted ref: path=" << path << " ref=" << ref;
+        qCDebug(category) << "Extracted ref: path=" << path << " ref=" << ref;
       }
     }
   }
@@ -347,9 +343,9 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
     int len = path.length();
     if( (len==1 && path[0]=='.') || (len==2 && path[0]=='.' && path[1]=='.') )
         path += '/';
-    //qCDebug(category) << "adding " << abs << " and " << path;
+    qCDebug(category) << "adding " << abs << " and " << path;
     abs = QDir::cleanPath(abs + '/' + path);
-    //qCDebug(category) << "checking whether " << abs << " exists.";
+    qCDebug(category) << "checking whether " << abs << " exists.";
     // Check if it exists
     if(QT_STAT(QFile::encodeName(abs), &buff) == 0) {
       path = abs; // yes -> store as the new cmd
@@ -368,11 +364,11 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
       if ( lastSlash > -1 && path.indexOf( ' ', lastSlash ) == -1 ) // no space after last slash, otherwise it's more likely command-line arguments
       {
         QString fileName = path.mid( lastSlash + 1 );
-        QString testPath = path.left( lastSlash + 1 );
+        QString testPath = path.left(lastSlash);
         if ((fileName.indexOf('*') != -1 || fileName.indexOf('[') != -1 || fileName.indexOf( '?' ) != -1)
                 && QT_STAT(QFile::encodeName(testPath), &buff) == 0) {
           nameFilter = fileName;
-          //qCDebug(category) << "Setting nameFilter to " << nameFilter;
+          qCDebug(category) << "Setting nameFilter to" << nameFilter << "and path to" << testPath;
           path = testPath;
           exists = true;
         }
@@ -384,7 +380,7 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
   if( exists )
   {
     QUrl u = QUrl::fromLocalFile(path);
-    //qCDebug(category) << "ref=" << stringDetails(ref) << " query=" << stringDetails(query);
+    qCDebug(category) << "ref=" << ref << "query=" << query;
     u.setFragment(ref);
     u.setQuery(query);
 
@@ -401,7 +397,7 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
     bool isDir = ((buff.st_mode & QT_STAT_MASK) == QT_STAT_DIR);
     if( !isDir && access ( QFile::encodeName(path).data(), X_OK) == 0 )
     {
-      //qCDebug(category) << "Abs path to EXECUTABLE";
+      qCDebug(category) << "Abs path to EXECUTABLE";
       setFilteredUri( data, u );
       setUriType( data, KUriFilterData::Executable );
       return true;
@@ -410,7 +406,7 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
     // Open "uri" as file:/xxx if it is a non-executable local resource.
     if( isDir || (( buff.st_mode & QT_STAT_MASK ) == QT_STAT_REG) )
     {
-      //qCDebug(category) << "Abs path as local file or directory";
+      qCDebug(category) << "Abs path as local file or directory";
       if ( !nameFilter.isEmpty() )
         u.setPath( u.path() + '/' + nameFilter );
       setFilteredUri( data, u );
@@ -429,11 +425,11 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
     // We try hard to avoid parsing any possible command
     // line arguments or options that might have been supplied.
     QString exe = removeArgs( cmd );
-    //qCDebug(category) << "findExe with" << exe;
+    qCDebug(category) << "findExe with" << exe;
 
     if (!QStandardPaths::findExecutable( exe ).isNull() )
     {
-      //qCDebug(category) << "EXECUTABLE  exe=" << exe;
+      qCDebug(category) << "EXECUTABLE  exe=" << exe;
       setFilteredUri( data, QUrl::fromLocalFile( exe ));
       // check if we have command line arguments
       if( exe != cmd )
@@ -448,7 +444,7 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
   // slow things down...
   if ( !isMalformed && !isLocalFullPath && !protocol.isEmpty() )
   {
-    //qCDebug(category) << "looking for protocol " << protocol;
+    qCDebug(category) << "looking for protocol" << protocol;
     if (isKnownProtocol(protocol))
     {
       setFilteredUri( data, url );
@@ -468,17 +464,15 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
     // TODO: Make configurable at some point...
     Q_FOREACH(const URLHint& hint, m_urlHints)
     {
+      qCDebug(category) << "testing regexp for" << hint.prepend;
       if (hint.regexp.indexIn(cmd) == 0)
       {
-        //qCDebug(category) << "match - prepending" << (*it).prepend;
         const QString cmdStr = hint.prepend + cmd;
         QUrl url(cmdStr);
-        //qCDebug(category) << "match - prepending" << hint.prepend << "->" << cmdStr << "->" << url;
-        if (isKnownProtocol(url.scheme())) {
-          setFilteredUri( data, url );
-          setUriType( data, hint.type );
-          return true;
-        }
+        qCDebug(category) << "match - prepending" << hint.prepend << "->" << cmdStr << "->" << url;
+        setFilteredUri( data, url );
+        setUriType( data, hint.type );
+        return true;
       }
     }
 
@@ -493,7 +487,7 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
       if (urlStr.isEmpty())
           urlStr = m_strDefaultUrlScheme;
 
-      const int index = urlStr.indexOf(QL1C(':'));
+      const int index = urlStr.indexOf(QLatin1Char(':'));
       if (index == -1 || !isKnownProtocol(urlStr.left(index)))
         urlStr += QStringLiteral("://");
       urlStr += cmd;
@@ -528,7 +522,7 @@ bool KShortUriFilter::filterUri( KUriFilterData& data ) const
       setUriType( data, KUriFilterData::LocalFile );
       return true;
     }
-    //qCDebug(category) << "fileNotFound -> ERROR";
+    qCDebug(category) << "fileNotFound -> ERROR";
     setErrorMsg( data, i18n( "<qt>The file or folder <b>%1</b> does not exist.</qt>", data.uri().toDisplayString() ) );
     setUriType( data, KUriFilterData::Error );
     return true;
@@ -558,6 +552,9 @@ void KShortUriFilter::configure()
   m_strDefaultUrlScheme = cg.readEntry( "DefaultProtocol", QStringLiteral("http://") );
   const EntryMap patterns = config.entryMap( QStringLiteral("Pattern") );
   const EntryMap protocols = config.entryMap( QStringLiteral("Protocol") );
+  if (cg.readEntry("Verbose", false)) {
+      category.setEnabled(QtDebugMsg, true);
+  }
   KConfigGroup typeGroup(&config, "Type");
 
   for( EntryMap::ConstIterator it = patterns.begin(); it != patterns.end(); ++it )
